#[non_exhaustive]pub enum PathOperation {
Move,
Close,
Line,
Quad,
Cubic,
Conic,
}Available on crate feature
v4_14 only.Expand description
Describes the segments of a Path.
More values may be added in the future.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
Move
A move-to operation, with 1 point describing the target point.
Close
A close operation ending the current contour with a line back to the starting point. Two points describe the start and end of the line.
Line
A line-to operation, with 2 points describing the start and end point of a straight line.
Quad
zier curve with 3 points describing the start point, the control point and the end point of the curve.
Cubic
zier curve with 4 points describing the start point, the two control points and the end point of the curve.
Conic
zier curve with 3 points describing the start point, control point and end point of the curve. A weight for the curve will be passed, too.