#[non_exhaustive]pub enum RenderNodeType {
Show 39 variants
NotARenderNode,
ContainerNode,
CairoNode,
ColorNode,
LinearGradientNode,
RepeatingLinearGradientNode,
RadialGradientNode,
RepeatingRadialGradientNode,
ConicGradientNode,
BorderNode,
TextureNode,
InsetShadowNode,
OutsetShadowNode,
TransformNode,
OpacityNode,
ColorMatrixNode,
RepeatNode,
ClipNode,
RoundedClipNode,
ShadowNode,
BlendNode,
CrossFadeNode,
TextNode,
BlurNode,
DebugNode,
GlShaderNode,
TextureScaleNode,
MaskNode,
FillNode,
StrokeNode,
SubsurfaceNode,
ComponentTransferNode,
CopyNode,
PasteNode,
CompositeNode,
IsolationNode,
DisplacementNode,
ArithmeticNode,
TurbulenceNode,
}Expand description
The type of a node determines what the node is rendering.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
NotARenderNode
Error type. No node will ever have this type.
ContainerNode
A node containing a stack of children
CairoNode
A node drawing a [cairo::Surface][crate::cairo::Surface]
ColorNode
A node drawing a single color rectangle
LinearGradientNode
A node drawing a linear gradient
RepeatingLinearGradientNode
A node drawing a repeating linear gradient
RadialGradientNode
A node drawing a radial gradient
RepeatingRadialGradientNode
A node drawing a repeating radial gradient
ConicGradientNode
A node drawing a conic gradient
BorderNode
A node stroking a border around an area
TextureNode
A node drawing a gdk::Texture
InsetShadowNode
A node drawing an inset shadow
OutsetShadowNode
A node drawing an outset shadow
TransformNode
A node that renders its child after applying a matrix transform
OpacityNode
A node that changes the opacity of its child
ColorMatrixNode
A node that applies a color matrix to every pixel
RepeatNode
A node that repeats the child’s contents
ClipNode
A node that clips its child to a rectangular area
RoundedClipNode
A node that clips its child to a rounded rectangle
ShadowNode
A node that draws a shadow below its child
BlendNode
A node that blends two children together
CrossFadeNode
A node that cross-fades between two children
TextNode
A node containing a glyph string
BlurNode
A node that applies a blur
DebugNode
Debug information that does not affect the rendering
GlShaderNode
A node that uses OpenGL fragment shaders to render
TextureScaleNode
A node drawing a gdk::Texture scaled and filtered.
MaskNode
A node that masks one child with another.
FillNode
A node that fills a path.
StrokeNode
A node that strokes a path.
SubsurfaceNode
A node that possibly redirects part of the scene graph to a subsurface.
ComponentTransferNode
A node that applies some function to each color component.
CopyNode
A node that copies the rendering canvas to be pasted later.
PasteNode
A node that pastes a previously copied canvas.
CompositeNode
A node that combines a child with the background using Porter/Duff operations.
IsolationNode
A node that isolated content of its child from previous content.
DisplacementNode
A node that displaces content according to some mask.
ArithmeticNode
A node that combines two child nodes in an arithmetic way.
TurbulenceNode
A node that generates a Perlin noise or fractal Brownian motion pattern.