pub struct ComponentTransferNode { /* private fields */ }v4_20 only.Expand description
A render node for applying a ComponentTransfer for each color
component of the child node.
GLib type: Shared boxed type with reference counted clone semantics.
Implementations
Sourceimpl ComponentTransferNode
impl ComponentTransferNode
Sourcepub fn as_ptr(&self) -> *mut GskComponentTransferNode
pub fn as_ptr(&self) -> *mut GskComponentTransferNode
Return the inner pointer to the underlying C value.
Sourcepub unsafe fn from_glib_ptr_borrow(ptr: &*mut GskComponentTransferNode) -> &Self
pub unsafe fn from_glib_ptr_borrow(ptr: &*mut GskComponentTransferNode) -> &Self
Borrows the underlying C value.
Sourceimpl ComponentTransferNode
impl ComponentTransferNode
Sourcepub fn new(
child: impl AsRef<RenderNode>,
r: &ComponentTransfer,
g: &ComponentTransfer,
b: &ComponentTransfer,
a: &ComponentTransfer,
) -> ComponentTransferNode
pub fn new( child: impl AsRef<RenderNode>, r: &ComponentTransfer, g: &ComponentTransfer, b: &ComponentTransfer, a: &ComponentTransfer, ) -> ComponentTransferNode
Sourcepub fn child(&self) -> RenderNode
pub fn child(&self) -> RenderNode
Sourcepub fn transfer(&self, component: ColorChannel) -> ComponentTransfer
Available on crate feature v4_22 only.
pub fn transfer(&self, component: ColorChannel) -> ComponentTransfer
v4_22 only.Methods from Deref<Target = RenderNode>
Sourcepub fn as_ptr(&self) -> *mut GskRenderNode
pub fn as_ptr(&self) -> *mut GskRenderNode
Return the inner pointer to the underlying C value.
pub const NONE: Option<&'static RenderNode> = None
Sourcepub fn draw(&self, cr: &Context)
pub fn draw(&self, cr: &Context)
Draws the contents of a render node on a cairo context.
Typically, you’ll use this function to implement fallback rendering
of render nodes on an intermediate Cairo context, instead of using
the drawing context associated to a gdk::Surface’s rendering buffer.
For advanced nodes that cannot be supported using Cairo, in particular for nodes doing 3D operations, this function may fail.
cr
cairo context to draw to
Sourcepub fn children(&self) -> Vec<RenderNode>
Available on crate feature v4_22 only.
pub fn children(&self) -> Vec<RenderNode>
v4_22 only.Gets a list of all children nodes of the rendernode.
Keep in mind that for various rendernodes, their children have different semantics, like the mask vs the source of a mask node. If you care about thse semantics, don’t use this function, use the specific getters instead.
Returns
The children
Sourcepub fn node_type(&self) -> RenderNodeType
pub fn node_type(&self) -> RenderNodeType
Sourcepub fn opaque_rect(&self) -> Option<Rect>
Available on crate feature v4_16 only.
pub fn opaque_rect(&self) -> Option<Rect>
v4_16 only.Gets an opaque rectangle inside the node that GTK can determine to be fully opaque.
There is no guarantee that this is indeed the largest opaque rectangle or that regions outside the rectangle are not opaque. This function is a best effort with that goal.
The rectangle will be fully contained in the bounds of the node.
Returns
true if part or all of the rendernode is opaque, false if no opaque region could be found.
out_opaque
return location for the opaque rect
Sourcepub fn serialize(&self) -> Bytes
pub fn serialize(&self) -> Bytes
Serializes the @self for later deserialization via gsk_render_node_deserialize(). No guarantees are made about the format used other than that the same version of GTK will be able to deserialize the result of a call to gsk_render_node_serialize() and gsk_render_node_deserialize() will correctly reject files it cannot open that were created with previous versions of GTK.
The intended use of this functions is testing, benchmarking and debugging. The format is not meant as a permanent storage format.
Returns
a GBytes representing the node.
Sourcepub fn write_to_file(&self, filename: impl AsRef<Path>) -> Result<(), Error>
pub fn write_to_file(&self, filename: impl AsRef<Path>) -> Result<(), Error>
This function is equivalent to calling serialize()
followed by file_set_contents().
See those two functions for details on the arguments.
It is mostly intended for use inside a debugger to quickly dump a render node to a file for later inspection.
filename
the file to save it to
Returns
true if saving was successful
pub fn is<T: IsRenderNode>(&self) -> bool
pub fn type_(&self) -> Type
pub fn downcast_ref<T: IsRenderNode>(&self) -> Option<&T>