pub struct Renderer { /* private fields */ }Expand description
Renders a scene graph defined via a tree of RenderNode instances.
Typically you will use a Renderer instance to repeatedly call
GskRendererExt::render() to update the contents of its associated
gdk::Surface.
It is necessary to realize a Renderer instance using
GskRendererExt::realize() before calling GskRendererExt::render(),
in order to create the appropriate windowing system resources needed
to render the scene.
This is an Abstract Base Class, you cannot instantiate it.
Properties
realized
Whether the renderer has been associated with a surface or draw context.
Readable
surface
The surface associated with renderer.
Readable
Implements
GLib type: GObject with reference counted clone semantics.
Implementations
Sourceimpl Renderer
impl Renderer
pub const NONE: Option<&'static Renderer> = None
Sourcepub fn for_surface(surface: &Surface) -> Option<Renderer>
pub fn for_surface(surface: &Surface) -> Option<Renderer>
Creates an appropriate Renderer instance for the given surface.
If the GSK_RENDERER environment variable is set, GSK will
try that renderer first, before trying the backend-specific
default. The ultimate fallback is the cairo renderer.
The renderer will be realized before it is returned.
surface
a surface
Returns
the realized renderer