#[non_exhaustive]pub enum SnapDirection {
None,
Floor,
Ceil,
Round,
}Available on crate feature
v4_24 only.Expand description
Specifies how a coordinate should be snapped to the pixel grid.
Note that the top and left sides of rectangles need to be snapped in the opposite direction from the bottom and right sides to make the rectangle grow or shrink.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
None
Don’t snap the value
Floor
Use floor() to snap
Ceil
Use ceil() to snap
Round
Use round() to snap